function SBexportSBMLGUI(model)
% SBexportSBMLGUI
% This function does the same as the "SBexportSBML" function, with the
% difference that it allows the user to edit the additional information
% using a GUI.
%
% USAGE:
% ======
% SBexportSBMLGUI(model)
%
% model: SBmodel to export to SBML
% 
% changes made within the GUI can be revoked by pressing the "Cancel"
% button or they can be assigned to the variable "changedModel" available
% in the MATLAB environment after clicking the "Exit" button

% Information:
% ============
% Author: Gunnar Drews, Student at University of Rostock Department of 
%         Computerscience, gunnar.drews@uni-rostock.de
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  
% USA.

text = sprintf('This function has been disabled in the current release.\nPlease use SBexportSBML instead. We will enable this function ASAP.');
error(text);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NO ALGEBRAIC RULES AND FAST FLAG SUPPORTED
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if usealgebraicSB(model),
    error('Algebraic rules not supported. Please use SBexportSBML.');
end
if usefastSB(model),
    error('Fast reactions not supported. Please use SBexportSBML.');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ADD JARS TO DYNAMIC CLASSPATH
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define all known parts of paths
if isunix,
    locationOfJarsInSBTOOLBOX = '/gui/SBexportSBML_dir/';
else
    locationOfJarsInSBTOOLBOX = '\gui\SBexportSBML_dir\';
end
path2SBTOOLBOX = fileparts(which('installSB'));
javamodelpath = char([double(path2SBTOOLBOX), double(locationOfJarsInSBTOOLBOX), double('javamodel.jar')]);
SBexportSBMLGUIpath = char([double(path2SBTOOLBOX), double(locationOfJarsInSBTOOLBOX), double('SBexportSBMLGUI.jar')]);
% suppress warnings for already sttic defined class paths
warning off MATLAB:javaclasspath:jarAlreadySpecified;
% add Jars to dynamic
javaaddpath(javamodelpath);
javaaddpath(SBexportSBMLGUIpath);
% turn warnings on again
warning on MATLAB:javaclasspath:jarAlreadySpecified;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK IF SBML Toolbox is present
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~exist('OutputSBML'),
    error('The SBML Toolbox is not installed on your system. You need it for the SBML export.');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK IF MODEL CONTAINS MATLAB FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(model.functionsMATLAB),
    error('Model contains MATLAB functions. Such functions are not supported in SBML.');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% test wether all name fields are filled and are unique
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
namesOK = checkNamesInSBmodel(model, 1);
if ~namesOK,
    errorMessage{1} = 'SBML export aborted because of errors concerning the names used in the given SBmodel. For further information use "checkNamesInSBmodel" function!';
    messageOutput(errorMessage, 1);
    return    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PREPROCESS and CHECK SBmodel (fill type, compartment, unittype fields if possible)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% check wether model is a Java SBmodel or a MATLAB SBmodel
model = SBmakeSBMLpresets(model);
testSBMLsettings(model, 1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Convert Model to Java Object and start GUI
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
javamodel = modelSB2Java(model);
SBexportSBMLGUI.SB2SBMLGUI(javamodel);

end