function [t,x,te,xe,ie,parameterValuesNew_ALL,F] = eventsimulateSB(model,method,tspan,ic,params,options,eventFunction,eventAssignmentFunction)
% eventsimulateSB: Function realizing the simulation of systems with events
% that cause discrete state changes. Implementation as auxiliary function
% in order to be able to reuse the code when simulating systems with events
% during parameter sensitivity analysis.
%
% 09/14/13 - updated to support output of ODE RHS for all time points.
% 09/16/13 - fixed a bug introduced in previous update in which most output
%            time points were erased when a vector of time points was
%            passed to eventsimulateSB.
% 
%
% Information:
% ============
% Copyright (C) 2005-2007 Fraunhofer Chalmers Centre, Gothenburg, Sweden
% Main author: Henning Schmidt
% 
% Changes for the SBTOOLBOX2:
% 1/1/2008  Henning Schmidt, henning@sbtoolbox2.org
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  
% USA.


parameterValuesNew = []; % for changing parameter values using events
parameterValuesNew_ALL = []; % save new parameter values for later generation of data ...
if isempty(params)
    nominalParameters = feval(model,'parametervalues')';
else
    nominalParameters = params';
    parameterValuesNew = params';               %testing this line, if problems are encountered with this fcn considering removing this line
%     parameterValuesNew = nominalParameters;
end

% initialize simulation results
t = [];
x = [];
te = [];
xe = [];
ie = [];
F = [];
% add the eventFunction to the integrator options
options = odeset(options,'Events',eventFunction);


% simulate the system until end time is reached
% in case the user provided a defined time instant vector when results are
% to be returned, it is necessary to delete 2 event instants from the
% vectors ...
% in case that the event time is not a measured time instant then delete
% the last element from the previous piece and the first element from the
% following piece
% in case that the event time is a member of tspan, the last two elements
% of the previous simulation data should be deleted. 
tspansim = tspan;
deleteFirstElementNext = 0;
while 1,
    % Do the simulation until event or finished
    parameterValuesNew = parameterValuesNew(:)';    
    sol = eval(sprintf('feval(@%s,@%s,tspansim,ic,options,parameterValuesNew);',method,model));
    tpiece = sol.x';
    xpiece = sol.y';
    tepiece = sol.xe;
    xepiece = sol.ye';
    iepiece = sol.ie;
    % get the state and final piece time
    pieceEndState = xpiece(end,:);
    pieceEndTime = tpiece(end);
    % check if an event has happened
    if ~isempty(tepiece),
        % an event has happened ... 
        eventTime = tepiece(end);   %all elements should have the same value
        eventIndex = iepiece;       %fixed critical bug - eventIndex used to take only the final element of iepiece  
        % just delete the last element (it's the event time anyway)
        if eventTime ~= max(tspan),        
            tpiece = tpiece(1:end-1);
            xpiece = xpiece(1:end-1,:);
        end
    end
    if length(tspan) > 2,
        % time vector given ...
        % evaluate solution at desired time points
        lb = min(tpiece);
        ub = max(tpiece);
        tpiece = tspan(tspan >= lb & tspan <= ub)';
        xpiece = deval(sol,tpiece);
        xpiece = xpiece';
    end
    % collect simulation data
    t = [t; tpiece];
    x = [x; xpiece];
    te = [te; tepiece(:)];
    ie = [ie; iepiece(:)];
    xe = [xe; xepiece];
    if nargout == 7
        [~,Fpiece] = deval(sol,sol.x);
        F = [F ; Fpiece'];
    end
    % save parameter values for this piece
    %parameterValuesNew is always empty during the first iteration
    if isempty(parameterValuesNew),
        parameterValuesNew_ALL = [parameterValuesNew_ALL; nominalParameters(ones(1,length(tpiece)),:)];
    else
        parameterValuesNew_ALL = [parameterValuesNew_ALL; parameterValuesNew(ones(1,length(tpiece)),:)];
    end

    % check if integration is finished - then break the loop and continue
    if pieceEndTime >= max(tspansim),
%         parameterValuesNew_ALL = [parameterValuesNew_ALL; nominalParameters(ones(1,length(tpiece)),:)];
        break;
    end
%     parameterValuesNew_ALL = [parameterValuesNew_ALL; parameterValuesNew(ones(1,length(tpiece)),:)];
    % set new initial conditions and new time vector for simulation
    [ic,parameterValuesNew] = feval(eventAssignmentFunction,eventIndex,eventTime,pieceEndState,parameterValuesNew);
    % set new tspansim
    if length(tspansim) == 2,
        % if tspansim has 2 elements:
        tspansim = [eventTime tspansim(2)];
    else
        % if tspansim given as vector of time instants
        tspansimhelp = tspansim(tspansim > eventTime);
        tspansim = [eventTime tspansimhelp(:)'];
    end
end
return